﻿DROP VIEW IF EXISTS deleted_approvers_v;

DROP VIEW IF EXISTS role_policy_v;

DROP VIEW IF EXISTS role_policy_all_v;

CREATE VIEW deleted_approvers_v AS 
 SELECT DISTINCT
  approvers.id,
  approvers.current_approval_step,
  approvers.request_policy_id,
  request_pol.name AS request_policy_name,
  approvers.approval_policy_id,
  approval_pol.name AS approval_policy_name,
  approvers.sod_unique_id,
  approvers.sod_policy,
  approvers.sod_policy_owner,
  approvers.psodv_approval_start_time,
  CASE
    WHEN approvers.psodv_approval_start_time IS NOT NULL THEN approvers.sod_policy_owner
    ELSE approvers.approver
  END AS deleted_approver_unique_id,
  CASE
    WHEN (approvers.psodv_approval_start_time IS NOT NULL AND psod_su.id IS NOT NULL) THEN psod_su.display_name
    WHEN (approvers.psodv_approval_start_time IS NOT NULL AND psod_sg.id IS NOT NULL) THEN psod_sg.name
    WHEN (approvers.psodv_approval_start_time IS NULL AND su.id IS NOT NULL) THEN su.display_name
    ELSE sg.name
  END AS deleted_approver,
  CASE
    WHEN (approvers.psodv_approval_start_time IS NOT NULL AND psod_su.id IS NOT NULL)
      OR (approvers.psodv_approval_start_time IS NULL AND su.id IS NOT NULL) THEN true
    WHEN (approvers.psodv_approval_start_time IS NOT NULL AND psod_sg.id IS NOT NULL)
      OR (approvers.psodv_approval_start_time IS NULL AND sg.id IS NOT NULL) THEN false
    ELSE NULL
  END AS is_user,
  CASE
    WHEN (approvers.psodv_approval_start_time IS NOT NULL AND psod_su.id IS NOT NULL) THEN psod_su.timestamp
    WHEN (approvers.psodv_approval_start_time IS NOT NULL AND psod_sg.id IS NOT NULL) THEN psod_sg.timestamp
    WHEN (approvers.psodv_approval_start_time IS NULL AND su.id IS NOT NULL) THEN su.timestamp
    ELSE sg.timestamp
  END AS timestamp,
  approvers.unique_item_id,
  approvers.item_request_reason,
  approvers.item_type,
  approvers.add_item,
  approvers.unique_requester_id,
  approvers.request_status,
  approvers.unique_recipient_id,
  approvers.item_approval_status,
  approvers.creation_time,
  approvers.creation_time + interval '1' day * aas.timeout_period AS approval_expiration,
  aas.timeout_action,
  approvers.approval_start_time,
  approvers.last_status_update_time,
  approvers.status_changed_by
FROM (
  SELECT 
    row_number() OVER (PARTITION BY approvals_info.request_item_id ORDER BY approvals_info.request_item_id) AS step,
    json_array_elements_text(approvals_info.approvals::json->'approvers') approver,
    approvals_info.*
  FROM
  (
    SELECT
      resource_request_info.*,
      json_array_elements(resource_request_info.approval_strategy_json::json->'approvals') AS approvals
    FROM (
      SELECT DISTINCT
        rr.id,
        rr.approval_strategy_json,
        rr.current_approval_step,
        rr.unique_requester_id,
        rr.request_status,
        rr.approval_policy_id,
        rr.creatiON_time,
        rr.approval_start_time,
        rr.last_update_time,
        sodp.unique_policy_id AS sod_unique_id,
        sodp.name AS sod_policy,
        po.unique_owner_id AS sod_policy_owner,
        rr.psodv_approval_start_time,
        rri.id AS request_item_id,
        rri.request_policy_id,
        rri.unique_item_id,
        rri.reason AS item_request_reason,
        rri.item_type,
        rri.add_item,
        rri.unique_recipient_id,
        rri.item_approval_status,
        rri.last_status_update_time,
        rri.status_changed_by
      FROM resource_request_item rri
      JOIN resource_request rr ON rr.id = rri.resource_request_id AND rr.request_status IN ('APPROVING','APPROVING_PSODV')
      LEFT JOIN psodv_approval_task_item pati ON pati.resource_request_item_id = rri.id
      LEFT JOIN psodv_approval_task pat ON pat.id = pati.psodv_approval_task_id
      LEFT JOIN sod_policy sodp ON sodp.id = pat.policy_id
      LEFT JOIN policy_owner po ON po.sod_policy = sodp.id
      WHERE rri.item_approval_status IN ('NO_DECISION', 'TENTATIVE_APPROVED', 'TENTATIVE_DENIED','WAITING_PSODV_APPROVAL')
    ) resource_request_info
  ) approvals_info
) approvers
LEFT JOIN (SELECT 
    suser.id,
    suser.unique_user_id,
    suser.display_name,
    suser.effective,
    suser.deleted,
    COALESCE(suser.last_modified,
      (SELECT min(sversion."timestamp") AS min 
        FROM sversion
        JOIN data_producer dp ON sversion.data_producer = dp.id
        WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > suser.sversion)
    ) AS timestamp
  FROM suser) su ON su.unique_user_id = approvers.approver AND su.effective = true
LEFT JOIN (SELECT 
    suser.id,
    suser.unique_user_id,
    suser.display_name,
    suser.effective,
    suser.deleted,
    COALESCE(suser.last_modified,
      (SELECT min(sversion."timestamp") AS min 
        FROM sversion
        JOIN data_producer dp ON sversion.data_producer = dp.id
        WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > suser.sversion)
    ) AS timestamp
  FROM suser) psod_su ON psod_su.unique_user_id = approvers.sod_policy_owner AND psod_su.effective = true
LEFT JOIN (SELECT 
    sgroup.id,
    sgroup.unique_group_id,
    sgroup.name,
    sgroup.effective,
    sgroup.deleted,
    COALESCE(sgroup.last_modified,
      (SELECT min(sversion."timestamp") AS min 
        FROM sversion
        JOIN data_producer dp ON sversion.data_producer = dp.id
        WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > sgroup.sversion)
    ) AS timestamp
  FROM sgroup) sg ON sg.unique_group_id = approvers.approver AND sg.effective = true
LEFT JOIN (SELECT 
    sgroup.id,
    sgroup.unique_group_id,
    sgroup.name,
    sgroup.effective,
    sgroup.deleted,
    COALESCE(sgroup.last_modified,
      (SELECT min(sversion."timestamp") AS min 
        FROM sversion
        JOIN data_producer dp ON sversion.data_producer = dp.id
        WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > sgroup.sversion)
    ) AS timestamp
  FROM sgroup) psod_sg ON psod_sg.unique_group_id = approvers.sod_policy_owner AND psod_sg.effective = true
LEFT JOIN access_req_approval_policy approval_pol ON approval_pol.id = approvers.approval_policy_id
LEFT JOIN access_approval_step aas ON aas.policy_id = approval_pol.id AND aas.step_sequence = approvers.current_approval_step 
LEFT JOIN access_req_policy request_pol ON request_pol.id = approvers.request_policy_id
WHERE (approvers.step = approvers.current_approval_step OR approvers.current_approval_step IS NULL) AND
  ((approvers.psodv_approval_start_time IS NULL AND su.id IS NOT NULL AND su.deleted = true) OR
   (approvers.psodv_approval_start_time IS NULL AND sg.id IS NOT NULL AND sg.deleted = true) OR
   (approvers.psodv_approval_start_time IS NULL AND su.id IS NULL AND sg.id IS NULL) OR
   (approvers.psodv_approval_start_time IS NOT NULL AND psod_su.id IS NOT NULL AND psod_su.deleted = true) OR
   (approvers.psodv_approval_start_time IS NOT NULL AND psod_sg.id IS NOT NULL AND psod_sg.deleted = true) OR
   (approvers.psodv_approval_start_time IS NOT NULL AND psod_su.id IS NULL AND psod_sg.id IS NULL));

CREATE VIEW role_policy_v AS 
 SELECT role_policy.id,
    role_policy.unique_policy_id,
    role_policy.deleted,
    role_policy.state,
    role_policy.description,
    role_policy.name,
    role_policy.risk,
    role_policy.created_by,
    role_policy.changed_by,
    role_policy.update_time,
    role_policy.creation_time,
    role_policy.detection_status,
    role_policy.risk_mode,
    role_policy.cost
   FROM role_policy
  WHERE role_policy.deleted = false;

  CREATE VIEW role_policy_all_v AS 
  SELECT role_policy.id,
    role_policy.unique_policy_id,
    role_policy.deleted,
    role_policy.state,
    role_policy.description,
    role_policy.name,
    role_policy.risk,
    role_policy.created_by,
    role_policy.changed_by,
    role_policy.update_time,
    role_policy.creation_time,
    role_policy.detection_status,
    role_policy.risk_mode,
    role_policy.cost
  FROM role_policy;

ALTER TABLE deleted_approvers_v
OWNER TO igops;
GRANT ALL ON TABLE deleted_approvers_v TO igops;
GRANT SELECT ON TABLE deleted_approvers_v TO ig_report_role;

ALTER TABLE role_policy_v
OWNER TO igops;
GRANT ALL ON TABLE role_policy_v TO igops;
GRANT SELECT ON TABLE role_policy_v TO ig_report_role;

ALTER TABLE role_policy_all_v
OWNER TO igops;
GRANT ALL ON TABLE role_policy_all_v TO igops;
GRANT SELECT ON TABLE role_policy_all_v TO ig_report_role;